@@ -154,6 +154,22 @@ local function kvs_to_node(kvs)
 end
 _M.kvs_to_node = kvs_to_node
 
+local function kvs_to_nodes(res, exclude_dir)
+    res.body.node.dir = true
+    res.body.node.nodes = setmetatable({}, array_mt)
+    if exclude_dir then
+        for i=2, #res.body.kvs do
+            res.body.node.nodes[i-1] = kvs_to_node(res.body.kvs[i])
+        end
+    else
+        for i=1, #res.body.kvs do
+            res.body.node.nodes[i] = kvs_to_node(res.body.kvs[i])
+        end
+    end
+    return res
+end
+
+
 local function not_found(res)
     res.body.message = "Key not found"
     res.reason = "Not found"
@@ -201,22 +217,23 @@ function _M.get_format(res, real_key, is_dir, formatter)
     else
         -- In etcd v2, the direct key asked for is `node`, others which under this dir are `nodes`
         -- While in v3, this structure is flatten and all keys related the key asked for are `kvs`
-        res.body.node = {
-            key = real_key,
-            dir = true,
-            nodes = setmetatable({}, array_mt)
-        }
-        local kvs = res.body.kvs
-        if #kvs >= 1 and not kvs[1].value then
-            res.body.node.createdIndex = tonumber(kvs[1].create_revision)
-            res.body.node.modifiedIndex = tonumber(kvs[1].mod_revision)
-            for i=2, #kvs do
-                res.body.node.nodes[i-1] = kvs_to_node(kvs[i])
+        res.body.node = kvs_to_node(res.body.kvs[1])
+        -- we have a init_dir (for etcd v2) value that can't be deserialized with json,
+        -- but we don't put init_dir for new resource type like consumer credential
+        if not res.body.kvs[1].value then
+            -- remove last "/" when necessary
+            if string.byte(res.body.node.key, -1) == 47 then
+                res.body.node.key = string.sub(res.body.node.key, 1, #res.body.node.key-1)
             end
+            res = kvs_to_nodes(res, true)
         else
-            for i=1, #kvs do
-                res.body.node.nodes[i] = kvs_to_node(kvs[i])
+            -- get dir key by remove last part of node key,
+            -- for example: /apisix/consumers/jack -> /apisix/consumers
+            local last_slash_index = string.find(res.body.node.key, "/[^/]*$")
+            if last_slash_index then
+                res.body.node.key = string.sub(res.body.node.key, 1, last_slash_index-1)
             end
+            res = kvs_to_nodes(res, false)
         end
     end
 
@@ -484,6 +501,30 @@ function _M.delete(key)
     return res, nil
 end
 
+function _M.rmdir(key, opts)
+    local etcd_cli, prefix, err = get_etcd_cli()
+    if not etcd_cli then
+        return nil, err
+    end
+
+    local res, err = etcd_cli:rmdir(prefix .. key, opts)
+    if not res then
+        return nil, err
+    end
+
+    res.headers["X-Etcd-Index"] = res.body.header.revision
+
+    if not res.body.deleted then
+        return not_found(res), nil
+    end
+
+    v3_adapter.to_v3(res.body, "delete")
+    res.body.node = {}
+    res.body.key = prefix .. key
+
+    return res, nil
+end
+
 ---
 -- Get etcd cluster and server version.
 --
