@@ -26,7 +26,6 @@ local byte      = string.byte
 local type      = type
 local pcall     = pcall
 local pairs     = pairs
-local ipairs    = ipairs
 
 local _M = {}
 
@@ -50,34 +49,6 @@ local function check_secret(conf)
 end
 
 
-local secret_kv_lrucache = core.lrucache.new({
-    ttl = 300, count = 512
-})
-
-local function create_secret_kvs(values)
-    local secret_managers = {}
-
-    for _, v in ipairs(values) do
-        local path = v.value.id
-        local idx = find(path, "/")
-        if not idx then
-            core.log.error("no secret id")
-            return nil
-        end
-
-        local manager = sub(path, 1, idx - 1)
-        local id = sub(path, idx + 1)
-
-        if not secret_managers[manager] then
-            secret_managers[manager] = {}
-        end
-        secret_managers[manager][id] = v.value
-    end
-
-    return secret_managers
-end
-
-
  local function secret_kv(manager, confid)
     local secret_values
     secret_values = core.config.fetch_created_obj("/secrets")
@@ -85,9 +56,12 @@ end
        return nil
     end
 
-    local secret_managers = secret_kv_lrucache("secret_kv", secret_values.conf_version,
-                create_secret_kvs, secret_values.values)
-    return secret_managers[manager] and secret_managers[manager][confid]
+    local secret = secret_values:get(manager .. "/" .. confid)
+    if not secret then
+        return nil
+    end
+
+    return secret.value
 end
 
 
@@ -161,6 +135,7 @@ end
 
 
 local function fetch_by_uri(secret_uri)
+    core.log.info("fetching data from secret uri: ", secret_uri)
     local opts, err = parse_secret_uri(secret_uri)
     if not opts then
         return nil, err
