@@ -110,8 +110,6 @@ do
                     for _, route in ipairs(items) do
                         local hit = match_addrs(route, vars)
                         if hit then
-                            route.value.remote_addr_matcher = nil
-                            route.value.server_addr_matcher = nil
                             ctx.matched_route = route
                             return true
                         end
@@ -177,8 +175,6 @@ do
         for _, route in ipairs(other_routes) do
             local hit = match_addrs(route, api_ctx.var)
             if hit then
-                route.value.remote_addr_matcher = nil
-                route.value.server_addr_matcher = nil
                 api_ctx.matched_route = route
                 return true
             end
